"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.composeAsyncStatusAutomationState = exports.createAsyncStatusesDimension = void 0;
var createBaseDimension_1 = require("./createBaseDimension");
var types_1 = require("./types");
var getAsyncStatusInitialState = function (asyncActions) {
    var asyncStatuses = {};
    Object.keys(asyncActions).forEach(function (key) {
        var typedKey = key;
        asyncStatuses[typedKey] = {
            status: types_1.asyncLifecycleStatuses.NO_CURRENT_CALL,
            previousStatus: types_1.asyncLifecycleStatuses.NO_CURRENT_CALL,
            isPending: false,
            hasError: false,
        };
    });
    return asyncStatuses;
};
var getAsyncStatusReducers = function (state) {
    return {
        setAsyncStatus: function (_a) {
            var _b;
            var callName = _a.callName, status = _a.status;
            var typedCallName = callName;
            return __assign(__assign({}, state), (_b = {}, _b[typedCallName] = __assign(__assign({}, state[typedCallName]), { status: status, previousStatus: state[typedCallName].status, isPending: status === types_1.asyncLifecycleStatuses.PENDING, hasError: status === types_1.asyncLifecycleStatuses.CALL_FAILURE }), _b));
        }
    };
};
var createAsyncStatusesDimension = function (originalStoreKey, asyncActions, externalDependencies) {
    var dimensionStoreKey = originalStoreKey + "_Status";
    var asyncStatusesDimensionInitialState = getAsyncStatusInitialState(asyncActions);
    var statusesDimension = (0, createBaseDimension_1.createBaseDimension)(dimensionStoreKey, function () { return asyncStatusesDimensionInitialState; }, function () { return getAsyncStatusReducers; }, externalDependencies);
    return statusesDimension;
};
exports.createAsyncStatusesDimension = createAsyncStatusesDimension;
var composeAsyncStatusAutomationState = function (asyncActions, asyncStatusesDimension) {
    var higherOrderAsyncActions = {};
    var reducers = asyncStatusesDimension.reducers;
    Object.keys(asyncActions).forEach(function (key) {
        higherOrderAsyncActions[key] = function (params) {
            reducers.setAsyncStatus({ callName: key, status: types_1.asyncLifecycleStatuses.PENDING });
            return asyncActions[key](params)
                .then(function (result) {
                reducers.setAsyncStatus({ callName: key, status: types_1.asyncLifecycleStatuses.CALL_SUCCESS });
                return result;
            })
                .catch(function (error) {
                reducers.setAsyncStatus({ callName: key, status: types_1.asyncLifecycleStatuses.CALL_FAILURE });
                return error;
            })
                .finally(function () {
                reducers.setAsyncStatus({ callName: key, status: types_1.asyncLifecycleStatuses.NO_CURRENT_CALL });
            });
        };
    });
    return higherOrderAsyncActions;
};
exports.composeAsyncStatusAutomationState = composeAsyncStatusAutomationState;
