"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createBaseDimension = void 0;
var react_redux_1 = require("react-redux");
var storeRegistry_1 = require("./storeRegistry");
var dimensionStoreRegistry_1 = require("./dimensionStoreRegistry");
var mapReducersToStoreDispatch = function (dimensionStoreKey, reducers, callback) {
    var mappedReducer = {};
    Object.keys(reducers).forEach(function (key) {
        var newKey = key.replace(dimensionStoreKey + "_/_", '');
        mappedReducer[newKey] = callback(key);
    });
    return mappedReducer;
};
var initializeDimensionReducers = function (dimensionStoreKey, initialState, reducers, store) {
    var baseReducers = {};
    store.injectReducer(dimensionStoreKey, function (state, _a) {
        if (state === void 0) { state = initialState; }
        var type = _a.type, payload = _a.payload;
        var stateEnhancedReducers = reducers(state);
        Object.keys(stateEnhancedReducers).forEach(function (key) {
            baseReducers[dimensionStoreKey + "_/_" + key] = stateEnhancedReducers[key];
        });
        return baseReducers[type] ? baseReducers[type](payload) : state;
    });
    return baseReducers;
};
/**
 * @param dimensionStoreKey represents the name of the dimension of the redux store
 *
 * @param initialStateClosure represents the initial state of the given dimension
 *
 * @param getDimensionReducersFunction is a function that passes state to a series of mapped reducers
 *
 * @param dimensionAsyncActions is a map of asynchronous actions with their properties specified
 *
 * @returns an object representing the store dimension properties
 */
var createBaseDimension = function (dimensionStoreKey, initialStateClosure, reducersClosure, externalDependencies, store) {
    if (store === void 0) { store = (0, storeRegistry_1.getStore)(); }
    var initialState = initialStateClosure(externalDependencies);
    var reducers = reducersClosure(externalDependencies);
    var dimensionReducers = initializeDimensionReducers(dimensionStoreKey, initialState, reducers, store);
    var dimensionStore = (0, dimensionStoreRegistry_1.createDimensionStore)(store, dimensionStoreKey);
    var dimensionState = function () { return (0, react_redux_1.useSelector)(function (storeState) { return storeState[dimensionStoreKey]; }); };
    var dispatchedDimensionReducers = mapReducersToStoreDispatch(dimensionStoreKey, dimensionReducers, function (type) { return function (payload) {
        store.dispatch({ type: type, payload: payload });
    }; });
    var baseDimension = {
        use: dimensionState,
        reducers: dispatchedDimensionReducers,
        store: dimensionStore,
    };
    return baseDimension;
};
exports.createBaseDimension = createBaseDimension;
