"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createDimension = void 0;
var storeRegistry_1 = require("./storeRegistry");
var createBaseDimension_1 = require("./createBaseDimension");
var asyncStatusAutomationHelpers_1 = require("./asyncStatusAutomationHelpers");
/**
 * @param dimensionStoreKey represents the name of the dimension of the redux store
 *
 * @param initialStateClosure represents the initial state of the given dimension
 *
 * @param getDimensionReducersFunction is a function that passes state to a series of mapped reducers
 *
 * @param dimensionAsyncActions is a map of asynchronous actions with their properties specified
 *
 * @returns an object representing the store dimension properties
 */
var createDimension = function (dimensionParamers) {
    if (dimensionParamers.store === undefined)
        dimensionParamers.store = (0, storeRegistry_1.getStore)();
    if (dimensionParamers.addAsyncStatusAutomationState === undefined)
        dimensionParamers.addAsyncStatusAutomationState = true;
    var dimensionStoreKey = dimensionParamers.dimensionStoreKey, initialStateClosure = dimensionParamers.initialStateClosure, reducersClosure = dimensionParamers.reducersClosure, selectorsClosure = dimensionParamers.selectorsClosure, asyncActionsClosure = dimensionParamers.asyncActionsClosure, executorsClosure = dimensionParamers.executorsClosure, externalDependencies = dimensionParamers.externalDependencies, addAsyncStatusAutomationState = dimensionParamers.addAsyncStatusAutomationState, store = dimensionParamers.store;
    var baseDimension = (0, createBaseDimension_1.createBaseDimension)(dimensionStoreKey, initialStateClosure, reducersClosure, externalDependencies, store);
    var asyncStatusesDimension;
    var executors = {};
    var selectors = selectorsClosure(__assign({ dimension: baseDimension }, externalDependencies));
    var asyncActions = asyncActionsClosure(__assign({ dimension: __assign(__assign({}, baseDimension), { selectors: selectors }) }, externalDependencies));
    if (addAsyncStatusAutomationState) {
        asyncStatusesDimension = (0, asyncStatusAutomationHelpers_1.createAsyncStatusesDimension)(dimensionStoreKey, asyncActions, externalDependencies);
        asyncActions = (0, asyncStatusAutomationHelpers_1.composeAsyncStatusAutomationState)(asyncActions, asyncStatusesDimension);
    }
    var useAsyncStatuses = asyncStatusesDimension ? asyncStatusesDimension.use : undefined;
    if (executorsClosure)
        executors = executorsClosure(__assign({ dimension: __assign(__assign({}, baseDimension), { selectors: selectors, asyncActions: asyncActions, useAsyncStatuses: useAsyncStatuses }) }, externalDependencies));
    return __assign(__assign({}, baseDimension), { selectors: selectors, asyncActions: asyncActions, executors: executors, useAsyncStatuses: useAsyncStatuses });
};
exports.createDimension = createDimension;
