"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createStore = exports.setStore = exports.getStore = void 0;
var redux_1 = require("redux");
var store;
/**
 * @returns the current or newly created store
 */
var getStore = function () {
    if (store)
        return store;
    store = createStore();
    return store;
};
exports.getStore = getStore;
/**
 * Call setStore to provide your own store for dimension-state to use. You'll need to use this if you want to use middleware.
 *
 * @param store store to set in the store-registry
 *
 * @returns the current store
 */
var setStore = function (initialStore) {
    if (store != null) {
        console.warn('Store is already initialized. Call setStore before the first getStore. This call will be ignored.');
        return null;
    }
    if (typeof initialStore.injectReducer !== 'function')
        throw new Error('Store must support .injectReducer');
    store = initialStore;
    return store;
};
exports.setStore = setStore;
/**
 * Creates a Redux store that holds the state tree.
 *
 * @param initialReducers An object whose values correspond to different reducer functions that already exist in the app.
 *
 * @param enhancer An optional extension for adding third-party capabilities such as middleware and time travel
 *
 * @returns A Redux store that lets you read the state, dispatch actions and subscribe to changes.
 */
var createStore = function (initialReducers, enhancer) {
    if (typeof initialReducers !== 'object')
        throw new Error('initialReducers should be an object suitable to be passed to combineReducers');
    var reducers = __assign(__assign({}, initialReducers), { _stub_: function (s) { return s || 0; } });
    var store = (0, redux_1.createStore)((0, redux_1.combineReducers)(reducers), enhancer);
    store.injectReducer = function (key, reducer) {
        if (reducers[key])
            // eslint-disable-next-line no-console
            console.warn("injectReducer: replacing reducer for key '" + key + "'");
        reducers[key] = reducer;
        store.replaceReducer((0, redux_1.combineReducers)(reducers));
    };
    Object.keys(reducers).forEach(function (key) {
        if (key !== '_stub_')
            store.injectReducer(key, reducers[key]);
    });
    return store;
};
exports.createStore = createStore;
